@@ -1,4 +1,4 @@
-#!/usr/bin/ruby
+#!/usr/bin/env ruby
 
 require 'xmms'
 
@@ -34,6 +34,10 @@ xr.playlist.each { |entry|
   puts "'#{title}', #{time}, #{file}"
 }
 
+# test playlist block method
+$stderr.puts 'testing Xmms::Remote#playlist (block)'
+xr.playlist { |title, file, time| puts "'#{title}', #{time}, #{file}" }
+
 # test [] method
 $stderr.puts 'testing Xmms::Remote#[]'
 1.upto(5) { |i|
