@@ -30,7 +30,7 @@
 
 #define UNUSED(x)  ((void) (x))
 
-#define VERSION "0.1.1"
+#define VERSION "0.1.2"
 #define NUM_BANDS 10
 #define BAND_MAX 20.0
 #define BAND_MIN -20.0
@@ -41,11 +41,15 @@
 /* CLASS AND MODULE GLOBALS */
 /****************************/
 static VALUE mXmms,
-             cRemote;
+             cRemote,
+             eError;
 
 /*
  * Create a new Xmms::Remote object.
  *
+ * This method raises an ArgumentError exception if the number of
+ * arguments isn't 0 or 1.
+ *
  * Examples:
  *   # standard setup (one running copy of XMMS)
  *   remote = Xmms::Remote.new
@@ -90,6 +94,8 @@ static VALUE xr_init(int argc, VALUE *argv, VALUE self) {
   return self;
 }
 
+#define CHECK_SESSION(session) if (!xmms_remote_is_running(*session)) rb_raise(eError, "XMMS is not running")
+
 /*
  * Get the version of XMMS.
  *
@@ -104,7 +110,8 @@ static VALUE xr_version(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
-
+  CHECK_SESSION(session);
+  
   return INT2FIX(xmms_remote_get_version(*session));
 }
 
@@ -115,6 +122,8 @@ static VALUE xr_version(VALUE self) {
 /*
  * Play current song.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Example:
  *   remote.play
  *
@@ -123,6 +132,7 @@ static VALUE xr_play(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_play(*session);
 
   return self;
@@ -131,6 +141,8 @@ static VALUE xr_play(VALUE self) {
 /*
  * Pause current song.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Example:
  *   remote.pause
  *
@@ -139,6 +151,7 @@ static VALUE xr_pause(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_pause(*session);
 
   return self;
@@ -147,6 +160,8 @@ static VALUE xr_pause(VALUE self) {
 /*
  * Stop current song.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Example:
  *   remote.stop
  *
@@ -155,6 +170,7 @@ static VALUE xr_stop(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_stop(*session);
 
   return self;
@@ -163,6 +179,8 @@ static VALUE xr_stop(VALUE self) {
 /*
  * XMMS eject button (toggle add file dialog).
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Example:
  *   remote.eject
  *
@@ -171,6 +189,7 @@ static VALUE xr_eject(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_eject(*session);
 
   return self;
@@ -179,6 +198,8 @@ static VALUE xr_eject(VALUE self) {
 /*
  * Quit XMMS.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Example:
  *   remote.quit
  *
@@ -187,6 +208,7 @@ static VALUE xr_quit(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_quit(*session);
 
   return self;
@@ -195,6 +217,8 @@ static VALUE xr_quit(VALUE self) {
 /*
  * Toggle Play/Pause status.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Example:
  *   remote.play_pause
  *
@@ -203,6 +227,7 @@ static VALUE xr_play_pause(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_play_pause(*session);
 
   return self;
@@ -215,6 +240,8 @@ static VALUE xr_play_pause(VALUE self) {
 /*
  * Is XMMS playing?
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   puts 'playing' if remote.is_playing?
  *   puts 'playing' if remote.playing?
@@ -224,6 +251,7 @@ static VALUE xr_playing(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   return xmms_remote_is_playing(*session) ? Qtrue : Qfalse;
 }
@@ -231,6 +259,8 @@ static VALUE xr_playing(VALUE self) {
 /*
  * Is XMMS paused?
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   puts 'paused' if remote.is_paused?
  *   puts 'paused' if remote.paused?
@@ -240,6 +270,7 @@ static VALUE xr_paused(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   return xmms_remote_is_paused(*session) ? Qtrue : Qfalse;
 }
@@ -249,38 +280,69 @@ static VALUE xr_paused(VALUE self) {
 /********************/
 
 /*
- * Return the current playlist.
+ * Return the current playlist.  If a block is given, pass each playlist
+ * element to the block.
+ *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
+ * Note: Returning the full array can be very slow for large playlists;
+ * use the block form instead.
  *
  * Example:
- *   remote.playlist.each { |i|
- *     title, file = i
- *     puts "'#{title}', #{file}"
- *   }
+ *   # print out info for each element in the playlist
+ *   ary = remote.playlist
+ *   ary.each do |ary|
+ *     title, file, time = ary
+ *     puts "'#{title}' (#{time}ms), #{file}"
+ *   end
+ *
+ *   # print out info for each element in the playlist (using block 
+ *   # syntax; much faster)
+ *   remote.playlist { |e| puts "'#{e[0]}' (#{e[2]}), #{e[1]}" }
  *
  */
 static VALUE xr_pl(VALUE self) {
   int i, len, *session;
-  VALUE e, ary;
+  VALUE e, ret;
+  char block_given = 0;
   
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
-  ary = rb_ary_new();
+  block_given = rb_block_given_p();
+  ret = block_given ? Qnil : rb_ary_new();
   len = xmms_remote_get_playlist_length(*session);
 
+  e = Qnil;
   for (i = 0; i < len; i++) {
-    e = rb_ary_new();
+    /* if we're yielding to a block, pass the same array each time; this 
+     * should speed things up (by saving on allocations) */
+    if (!block_given || e == Qnil)
+      e = rb_ary_new();
+    else
+      e = rb_ary_clear(e);
+
+    /* add info for current playlist element to array */
     rb_ary_push(e, rb_str_new2(xmms_remote_get_playlist_title(*session, i)));
     rb_ary_push(e, rb_str_new2(xmms_remote_get_playlist_file(*session, i)));
     rb_ary_push(e, INT2FIX(xmms_remote_get_playlist_time(*session, i)));
-    rb_ary_push(ary, e);
+
+    /* if block was given, yield current element; otherwise push it
+     * into the return array */
+    block_given ? rb_yield(e) : rb_ary_push(ret, e);
   }
 
-  return ary;
+  return ret;
 }
 
 /*
  * Add one or more songs to the playlist.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running,
+ * an ArgumentError exception if the number of arguments is less than 1,
+ * or a TypeError exception if the argument isn't a String, True, or
+ * False.
+ *
  * Examples:
  *   # replace the current playlist with one song
  *   enqueue = false
@@ -317,6 +379,7 @@ static VALUE xr_pl_add(int argc, VALUE *argv, VALUE self) {
   }
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_playlist(*session, list, max, enqueue);
 
   /* free the list, but not the elements */
@@ -328,6 +391,8 @@ static VALUE xr_pl_add(int argc, VALUE *argv, VALUE self) {
 /*
  * Add a URL to the current playlist.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   remote.add_url 'http://www.hhmecca.net/cool_song.mp3'
  *
@@ -336,6 +401,7 @@ static VALUE xr_pl_add_url(VALUE self, VALUE url) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_playlist_add_url_string(*session, RSTRING(url)->ptr);
 
   return self;
@@ -344,6 +410,8 @@ static VALUE xr_pl_add_url(VALUE self, VALUE url) {
 /*
  * Insert a URL into the current playlist.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   # add a url after the 45th element in the playlist
  *   remote.ins_url 'http://www.hhmecca.net/cool_song.mp3', 45
@@ -353,6 +421,7 @@ static VALUE xr_pl_ins_url(VALUE self, VALUE url, VALUE pos) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_playlist_ins_url_string(*session,
                                       RSTRING(url)->ptr,
                                       NUM2INT(pos));
@@ -363,6 +432,8 @@ static VALUE xr_pl_ins_url(VALUE self, VALUE url, VALUE pos) {
 /*
  * Delete the Nth element of the current playlist.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   remote.delete 26  # delete the 26th playlist element
  *
@@ -371,6 +442,7 @@ static VALUE xr_pl_del(VALUE self, VALUE pos) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_playlist_delete(*session, NUM2INT(pos));
 
   return self;
@@ -387,6 +459,7 @@ static VALUE xr_pl_clear(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_playlist_clear(*session);
 
   return self;
@@ -395,6 +468,8 @@ static VALUE xr_pl_clear(VALUE self) {
 /*
  * Get the current playlist position.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   song_number = remote.position
  *   song_number = remote.get_pos
@@ -405,6 +480,7 @@ static VALUE xr_pl_pos(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   return INT2FIX(xmms_remote_get_playlist_pos(*session));
 }
@@ -412,6 +488,8 @@ static VALUE xr_pl_pos(VALUE self) {
 /*
  * Set the current playlist position.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   remote.pos = 45
  *   remote.position = 45
@@ -422,6 +500,7 @@ static VALUE xr_pl_set_pos(VALUE self, VALUE pos) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_set_playlist_pos(*session, NUM2INT(pos));
 
   return self;
@@ -430,6 +509,10 @@ static VALUE xr_pl_set_pos(VALUE self, VALUE pos) {
 /*
  * Get the file path of a song.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running,
+ * or an ArgumentError exception if the number of arguments isn't 0 or
+ * 1.
+ *
  * Examples:
  *   # get the path of the current song
  *   path = remote.file
@@ -442,6 +525,7 @@ static VALUE xr_pl_file(int argc, VALUE *argv, VALUE self) {
   int pos, *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   switch (argc) {
     case 0:
@@ -460,6 +544,10 @@ static VALUE xr_pl_file(int argc, VALUE *argv, VALUE self) {
 /*
  * Get the title of a song.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running,
+ * or an ArgumentError exception if the number of arguments isn't 0 or
+ * 1.
+ *
  * Examples:
  *   # get the title of the current song
  *   path = remote.file
@@ -472,6 +560,7 @@ static VALUE xr_pl_title(int argc, VALUE *argv, VALUE self) {
   int pos, *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   switch (argc) {
     case 0:
@@ -488,13 +577,17 @@ static VALUE xr_pl_title(int argc, VALUE *argv, VALUE self) {
 }
 
 /*
- * Get the "playlist" time of a song.
+ * Get the duration of a song, in milliseconds
+ *
+ * This method raises an Xmms::Error exception if XMMS is not running,
+ * or an ArgumentError exception if the number of arguments isn't 0 or
+ * 1.
  *
  * Examples:
- *   # get the "playlist" time of the current song
+ *   # get the duration of the current song
  *   time = remote.playlist_time
  *
- *   # get the "playlist" time of song 23 in the playlist
+ *   # get the duration of song 23 in the playlist
  *   time = remote.playlist_time 23
  *
  */
@@ -502,6 +595,7 @@ static VALUE xr_pl_time(int argc, VALUE *argv, VALUE self) {
   int pos, *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   switch (argc) {
     case 0:
@@ -520,6 +614,8 @@ static VALUE xr_pl_time(int argc, VALUE *argv, VALUE self) {
 /*
  * Return information about a specific song in the playlist.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   title, file, time = remote[45] # get info about the 45th song
  *   info = remote.get_entry 45 # get info about the 45th song
@@ -530,6 +626,7 @@ static VALUE xr_pl_ary(VALUE self, VALUE pos) {
   VALUE ary;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   p = NUM2INT(pos);
   ary = rb_ary_new();
@@ -547,6 +644,8 @@ static VALUE xr_pl_ary(VALUE self, VALUE pos) {
 /*
  * Get the output time of the current song.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   time = remote.time
  *   time = remote.output_time
@@ -556,6 +655,7 @@ static VALUE xr_time(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   return INT2FIX(xmms_remote_get_output_time(*session));
 }
@@ -563,6 +663,8 @@ static VALUE xr_time(VALUE self) {
 /*
  * Jump to a specific time of the current song.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   remote.jump 45   # jump 45 seconds into the current song
  *   remote.time = 45 # jump 45 seconds into the current song 
@@ -572,6 +674,7 @@ static VALUE xr_jump(VALUE self, VALUE pos) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   xmms_remote_jump_to_time(*session, NUM2INT(pos));
 
@@ -581,6 +684,8 @@ static VALUE xr_jump(VALUE self, VALUE pos) {
 /*
  * Go to the previous song.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Example:
  *   remote.prev
  *
@@ -597,6 +702,8 @@ static VALUE xr_prev(VALUE self) {
 /*
  * Go to the next song.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Example:
  *   remote.next
  *
@@ -605,6 +712,7 @@ static VALUE xr_next(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_playlist_next(*session);
 
   return self;
@@ -617,6 +725,8 @@ static VALUE xr_next(VALUE self) {
 /*
  * Get the stereo volume level.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Example:
  *   left, right = remote.get_stereo_volume
  *   left, right = remote.stereo_volume
@@ -627,6 +737,7 @@ static VALUE xr_stereo_vol(VALUE self) {
   VALUE ary;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_get_volume(*session, &l, &r);
 
   ary = rb_ary_new();
@@ -639,6 +750,8 @@ static VALUE xr_stereo_vol(VALUE self) {
 /*
  * Get the main volume level.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Example:
  *   vol = remote.volume
  *   vol = remote.main_volume
@@ -649,6 +762,7 @@ static VALUE xr_main_vol(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   
   return INT2FIX(xmms_remote_get_main_volume(*session));
 }
@@ -656,6 +770,8 @@ static VALUE xr_main_vol(VALUE self) {
 /* 
  * Set the stereo volume level.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   remote.set_stereo_volume left, right
  *
@@ -667,6 +783,7 @@ static VALUE xr_set_stereo_vol(VALUE self, VALUE l, VALUE r) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_set_volume(*session, NUM2INT(l), NUM2INT(r));
 
   return self;
@@ -675,6 +792,8 @@ static VALUE xr_set_stereo_vol(VALUE self, VALUE l, VALUE r) {
 /*
  * Set the main volume level.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   remote.volume = 23
  *   remote.main_volume = 23
@@ -685,6 +804,7 @@ static VALUE xr_set_main_vol(VALUE self, VALUE vol) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_set_main_volume(*session, NUM2INT(vol));
 
   return self;
@@ -698,6 +818,8 @@ static VALUE xr_set_main_vol(VALUE self, VALUE vol) {
 /*
  * Get the balance level.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Note: as of XMMS 1.2.6 xmms_remote_get_balance() does not appear to
  * work correctly when XMMS is paused or stopped, so this function doesn't
  * either.
@@ -710,6 +832,7 @@ static VALUE xr_balance(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   
   return INT2FIX(xmms_remote_get_balance(*session));
 }
@@ -717,6 +840,8 @@ static VALUE xr_balance(VALUE self) {
 /*
  * Set the balance level.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Note: as of XMMS 1.2.6 xmms_remote_set_balance() does not appear to
  * work correctly when XMMS is paused or stopped, so this function doesn't
  * either.
@@ -729,6 +854,7 @@ static VALUE xr_set_balance(VALUE self, VALUE bal) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_set_balance(*session, NUM2INT(bal));
 
   return self;
@@ -741,6 +867,8 @@ static VALUE xr_set_balance(VALUE self, VALUE bal) {
 /*
  * Get the current skin file.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   skin = remote.get_skin
  *   skin = remote.skin
@@ -750,6 +878,7 @@ static VALUE xr_skin(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   return rb_str_new2(xmms_remote_get_skin(*session));
 }
@@ -757,6 +886,8 @@ static VALUE xr_skin(VALUE self) {
 /*
  * Set the current skin file.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   remote.set_skin 'path/to/cool_skin.zip'
  *   remote.skin = 'path/to/cool_skin.zip'
@@ -766,6 +897,7 @@ static VALUE xr_set_skin(VALUE self, VALUE skin) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_set_skin(*session, RSTRING(skin)->ptr);
 
   return self;
@@ -778,6 +910,8 @@ static VALUE xr_set_skin(VALUE self, VALUE skin) {
 /*
  * Set the visibility of the main window.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   remote.set_main_toggle true
  *   remote.main_visible = true
@@ -788,6 +922,7 @@ static VALUE xr_main_toggle(VALUE self, VALUE vis) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_main_win_toggle(*session, vis);
 
   return self;
@@ -796,6 +931,8 @@ static VALUE xr_main_toggle(VALUE self, VALUE vis) {
 /*
  * Get the visibility of the main window.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   puts 'main window visible' if remote.main_visible?
  *   puts 'main window visible' if remote.main?
@@ -805,6 +942,7 @@ static VALUE xr_is_main(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   return xmms_remote_is_main_win(*session) ? Qtrue : Qfalse;
 }
@@ -812,6 +950,8 @@ static VALUE xr_is_main(VALUE self) {
 /*
  * Set the visibility of the playlist window.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   remote.set_playlist_toggle true
  *   remote.playlist_visible = true
@@ -823,6 +963,7 @@ static VALUE xr_pl_toggle(VALUE self, VALUE vis) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_pl_win_toggle(*session, vis);
 
   return self;
@@ -831,6 +972,8 @@ static VALUE xr_pl_toggle(VALUE self, VALUE vis) {
 /*
  * Get the visibility of the playlist window.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   puts 'playlist window visible' if remote.playlist_visible?
  *   puts 'playlist window visible' if remote.pl_visible?
@@ -842,6 +985,7 @@ static VALUE xr_is_pl(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   return xmms_remote_is_pl_win(*session) ? Qtrue : Qfalse;
 }
@@ -849,6 +993,8 @@ static VALUE xr_is_pl(VALUE self) {
 /*
  * Set the visibility of the equalizer window.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   remote.set_equalizer_toggle true
  *   remote.equalizer_visible = true
@@ -860,6 +1006,7 @@ static VALUE xr_eq_toggle(VALUE self, VALUE vis) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_eq_win_toggle(*session, vis);
 
   return self;
@@ -868,6 +1015,8 @@ static VALUE xr_eq_toggle(VALUE self, VALUE vis) {
 /*
  * Get the visibility of the equalizer window.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   puts 'equalizer window visible' if remote.equalizer_visible?
  *   puts 'equalizer window visible' if remote.eq_visible?
@@ -879,6 +1028,7 @@ static VALUE xr_is_eq(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   return xmms_remote_is_eq_win(*session) ? Qtrue : Qfalse;
 }
@@ -886,6 +1036,8 @@ static VALUE xr_is_eq(VALUE self) {
 /*
  * Show the preferences dialog.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   remote.show_prefs
  *   remote.prefs
@@ -895,6 +1047,7 @@ static VALUE xr_prefs(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_show_prefs_box(*session);
 
   return self;
@@ -903,6 +1056,8 @@ static VALUE xr_prefs(VALUE self) {
 /*
  * Set the "always on top" flag.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   remote.set_always_on_top true
  *   remote.always_on_top = true
@@ -913,6 +1068,7 @@ static VALUE xr_set_aot(VALUE self, VALUE aot) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_toggle_aot(*session, aot);
 
   return self;
@@ -925,6 +1081,8 @@ static VALUE xr_set_aot(VALUE self, VALUE aot) {
 /*
  * Toggle the repeat flag.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Example:
  *   remote.toggle_repeat
  *
@@ -933,6 +1091,7 @@ static VALUE xr_toggle_repeat(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_toggle_repeat(*session);
 
   return self;
@@ -941,6 +1100,8 @@ static VALUE xr_toggle_repeat(VALUE self) {
 /*
  * Is the repeat flag set?
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   puts 'repeating' if remote.is_repeating?
  *   puts 'repeating' if remote.repeating?
@@ -952,6 +1113,7 @@ static VALUE xr_repeat(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   return xmms_remote_is_repeat(*session) ? Qtrue : Qfalse;
 }
@@ -959,6 +1121,8 @@ static VALUE xr_repeat(VALUE self) {
 /*
  * Toggle the shuffle flag.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Example:
  *   remote.toggle_shuffle
  *
@@ -967,6 +1131,7 @@ static VALUE xr_toggle_shuffle(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_toggle_shuffle(*session);
 
   return self;
@@ -975,6 +1140,8 @@ static VALUE xr_toggle_shuffle(VALUE self) {
 /*
  * Is the shuffle flag set?
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   puts 'shuffling' if remote.is_shuffling?
  *   puts 'shuffling' if remote.shuffling?
@@ -986,6 +1153,7 @@ static VALUE xr_shuffle(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   return xmms_remote_is_shuffle(*session) ? Qtrue : Qfalse;
 }
@@ -995,7 +1163,9 @@ static VALUE xr_shuffle(VALUE self) {
 /****************/
 
 /*
- * Get various about the current song.
+ * Get various information about the current song.
+ *
+ * This method raises an Xmms::Error exception if XMMS is not running.
  *
  * Returns an array of bitrate, frequency, and numchannels,
  * respectively.
@@ -1010,12 +1180,13 @@ static VALUE xr_info(VALUE self) {
   VALUE ary;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_get_info(*session, &rate, &freq, &nch);
 
   ary = rb_ary_new();
-  rb_ary_push(ary, NUM2INT(rate));
-  rb_ary_push(ary, NUM2INT(freq));
-  rb_ary_push(ary, NUM2INT(nch));
+  rb_ary_push(ary, INT2FIX(rate));
+  rb_ary_push(ary, INT2FIX(freq));
+  rb_ary_push(ary, INT2FIX(nch));
 
   return ary;
 }
@@ -1050,6 +1221,8 @@ static VALUE xr_running(VALUE self) {
  *
  *   [1.0, [0.0, -0.5, 0.9, 0.0, 0.0, 0.0, 0.2, 0.5, -0.1, 0.0]]
  * 
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   preamp, bands = remote.get_equalizer
  *   eq = remote.eq
@@ -1061,6 +1234,7 @@ static VALUE xr_eq(VALUE self) {
   gfloat preamp, *bands;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_get_eq(*session, &preamp, &bands);
 
   band_ary = rb_ary_new();
@@ -1077,6 +1251,8 @@ static VALUE xr_eq(VALUE self) {
 /*
  * Get the equalizer preamp value.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   preamp = remote.get_preamp
  *   preamp = remote.preamp
@@ -1086,6 +1262,7 @@ static VALUE xr_eq_preamp(VALUE self) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   return rb_float_new(xmms_remote_get_eq_preamp(*session));
 }
@@ -1093,8 +1270,11 @@ static VALUE xr_eq_preamp(VALUE self) {
 /*
  * Get an equalizer band value.
  *
- * Note that legal band indexes range from 0 to 9 instead of 1 to 10.
+ * Note: Legal band indices range from 0 to 9 instead of 1 to 10.
  * 
+ * This method raises an Xmms::Error exception if XMMS is not running,
+ * and an ArgumentError exception if the band index is out of range.
+ *
  * Examples:
  *   band = remote.get_band 2
  *   band = remote.band 8
@@ -1105,10 +1285,11 @@ static VALUE xr_eq_band(VALUE self, VALUE band) {
   int b;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   b = NUM2INT(band);
 
   if (b < 0 || b >= NUM_BANDS)
-    rb_raise(rb_eArgError, "band out of range (band < 0 or band >= 10)");
+    rb_raise(rb_eArgError, "band index out of range (band < 0 or band >= 10)");
 
   return rb_float_new(xmms_remote_get_eq_band(*session, b));
 }
@@ -1119,6 +1300,10 @@ static VALUE xr_eq_band(VALUE self, VALUE band) {
  * This method takes either a preamp value and 10 band values, or a
  * preamp value and an array of band values.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running,
+ * and an ArgumentError exception if the number of arguments isn't 2 or
+ * 11.
+ *
  * Examples:
  *   bands = [0.0, -0.5, 0.9, 0.0, 0.0, 0.0, 0.2, 0.5, -0.1, 0.0]
  *   remote.set_equalizer 0.0, bands
@@ -1144,6 +1329,7 @@ static VALUE xr_set_eq(int argc, VALUE *argv, VALUE self) {
   }
   
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_set_eq(*session, NUM2DBL(argv[0]), bands);
 
   return self;
@@ -1152,6 +1338,8 @@ static VALUE xr_set_eq(int argc, VALUE *argv, VALUE self) {
 /*
  * Set the equalizer preamp value.
  *
+ * This method raises an Xmms::Error exception if XMMS is not running.
+ *
  * Examples:
  *   remote.set_preamp 0.5
  *   remote.preamp = 0.5
@@ -1161,6 +1349,7 @@ static VALUE xr_eq_set_preamp(VALUE self, VALUE preamp) {
   int *session;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
   xmms_remote_set_eq_preamp(*session, NUM2DBL(preamp));
 
   return self;
@@ -1169,8 +1358,11 @@ static VALUE xr_eq_set_preamp(VALUE self, VALUE preamp) {
 /*
  * Set an equalizer band value.
  *
- * Note that legal band indexes range from 0 to 9 instead of 1 to 10.
+ * Note: legal band indices range from 0 to 9 instead of 1 to 10.
  * 
+ * This method raises an Xmms::Error exception if XMMS is not running,
+ * or an ArgumentError exception if the band index is out of range.
+ *
  * Example:
  *   remote.set_band 2, 0.5
  *
@@ -1180,10 +1372,11 @@ static VALUE xr_eq_set_band(VALUE self, VALUE band, VALUE val) {
   int b;
 
   Data_Get_Struct(self, int, session);
+  CHECK_SESSION(session);
 
   b = NUM2INT(band);
   if (b < 0 || b >= NUM_BANDS)
-    rb_raise(rb_eArgError, "band out of range (band < 0 or band >= 10)");
+    rb_raise(rb_eArgError, "band index out of range (band < 0 or band >= 10)");
 
   xmms_remote_set_eq_band(*session, b, NUM2DBL(val));
 
@@ -1445,4 +1638,9 @@ void Init_xmms(void) {
 
   rb_define_method(cRemote, "get_version", xr_version, 0);
   rb_define_alias(cRemote, "version", "get_version");
+
+  /**********************/
+  /* define Error class */
+  /**********************/
+  eError = rb_define_class_under(mXmms, "Error", rb_eStandardError);
 }
